-- SRAM Control
-- Designed by T.Iwata at DIGITALFILTER.COM 2006/07/29

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY SRAM_CTRL IS
	PORT	(
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		CAPTURE	: IN std_logic;
		DOWNLOAD	: IN std_logic;
		PACKET	: IN std_logic;
		STROBE	: IN std_logic;
		TRIGENB	: IN std_logic;
		TRIGFOUND	: IN std_logic;
		DUTDATA	: IN std_logic_vector(7 downto 0);
		READY	: OUT std_logic;
		PICDATA	: OUT std_logic_vector(7 downto 0);
		SRAMADDR	: OUT std_logic_vector(11 downto 0);
		SRAMDATA	: INOUT std_logic_vector(7 downto 0);
		SRAMLE_N	: OUT std_logic;
		SRAMHE_N	: OUT std_logic;
		SRAMWR_N	: OUT std_logic;
		SRAMRD_N	: OUT std_logic;
		SRAMCS_N	: OUT std_logic;
		DIVIDE	: IN std_logic_vector(1 downto 0);
		CAPENB	: OUT std_logic );
END SRAM_CTRL;

ARCHITECTURE structure OF SRAM_CTRL IS

	signal WrAddrCount	: std_logic_vector(11 downto 0);
	signal RdAddrCount	: std_logic_vector(11 downto 0);
	signal RdAddrCount_tmp	: std_logic_vector(11 downto 0);
	signal RdAddrCount_8b	: std_logic_vector(7 downto 0);
	signal PacketCount	: std_logic_vector(3 downto 0);
	signal capture_dly0	: std_logic;
	signal capture_dly1	: std_logic;
	signal capture_edge	: std_logic;
	signal capturing	: std_logic;
	signal download_dly0	: std_logic;
	signal download_dly1	: std_logic;
	signal download_edge	: std_logic;
	signal packet_dly0	: std_logic;
	signal packet_dly1	: std_logic;
	signal packet_edge	: std_logic;
	signal strobe_dly0	: std_logic;
	signal strobe_dly1	: std_logic;
	signal strobe_edge	: std_logic;
	signal VariableCount : std_logic_vector(9 downto 0);
	signal capture_enb	: std_logic;
	signal CoMax : std_logic_vector(9 downto 0);
	
BEGIN
-------------------------------------------------------------- 
   CoMax <=   "0000000000" when DIVIDE = "00" 
         else "0000001001" when DIVIDE = "01" -- bug fixed on 2006/07/29 (from 0000001010)
         else "0001100011" when DIVIDE = "10" -- bug fixed on 2006/07/29 (from 0001100100)
         else "1111100111"; -- bug fixed on 2006/07/29 (from 1111101000)

-------------------------------------------------------------- 
gen_VariableCount : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		VariableCount <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if(VariableCount = CoMax) then 
			VariableCount <= (others => '0');
		else 
			VariableCount <= VariableCount + 1;
		end if; 
	end if;  
end process;

-------------------------------------------------------------- 
   capture_enb <= '1' when VariableCount = 0 else '0';
   CAPENB <= capture_enb;
   
-------------------------------------------------------------- 
gen_capture_dly : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		capture_dly1 <= '0';
		capture_dly0 <= '0';
	elsif( CLK'event and CLK = '1' ) then
		capture_dly1 <= capture_dly0; 
		capture_dly0 <= CAPTURE; 
	end if;  
end process;

-------------------------------------------------------------- 
	capture_edge <= capture_dly0 and not capture_dly1;

-------------------------------------------------------------- 
gen_capturing : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		capturing <= '0';
	elsif( CLK'event and CLK = '1' ) then
		if(capture_edge = '1') then
		   capturing <= '1';
		elsif((TRIGENB = '0' and download_edge = '1') 
		   or (TRIGENB = '1' and TRIGFOUND = '1')) then 
		   capturing <= '0';
		end if; 
	end if;  
end process;

-------------------------------------------------------------- 
   READY <= not capturing;
   
-------------------------------------------------------------- 
gen_download_dly : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		download_dly1 <= '0';
		download_dly0 <= '0';
	elsif( CLK'event and CLK = '1' ) then
		download_dly1 <= download_dly0; 
		download_dly0 <= DOWNLOAD; 
	end if;  
end process;

-------------------------------------------------------------- 
	download_edge <= download_dly0 and not download_dly1;

-------------------------------------------------------------- 
gen_packet_dly : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		packet_dly1 <= '0';
		packet_dly0 <= '0';
	elsif( CLK'event and CLK = '1' ) then
		packet_dly1 <= packet_dly0; 
		packet_dly0 <= PACKET; 
	end if;  
end process;

-------------------------------------------------------------- 
	packet_edge <= packet_dly0 and not packet_dly1;

-------------------------------------------------------------- 
gen_strobe_dly : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		strobe_dly1 <= '0';
		strobe_dly0 <= '0';
	elsif( CLK'event and CLK = '1' ) then
		strobe_dly1 <= strobe_dly0; 
		strobe_dly0 <= STROBE; 
	end if;  
end process;

-------------------------------------------------------------- 
	strobe_edge <= strobe_dly0 and not strobe_dly1;
	
-------------------------------------------------------------- 
gen_WrAddrCount : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		WrAddrCount <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(capture_edge = '1') then 
			WrAddrCount <= (others => '1');
		elsif(capturing = '1' and capture_enb = '1') then 
			WrAddrCount <= WrAddrCount + 1;
		end if; 
	end if;  
end process;

-------------------------------------------------------------- 
gen_RdAddrCount_8b : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		RdAddrCount_8b <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(packet_edge = '1') then 
			RdAddrCount_8b <= (others => '1');
		elsif(strobe_edge = '1') then 
			RdAddrCount_8b <= RdAddrCount_8b + 1;
		end if; 
	end if;  
end process;

-------------------------------------------------------------- 
gen_PacketCount : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		PacketCount <= (others => '1'); 
	elsif( CLK'event and CLK = '1' ) then
		if(download_edge = '1') then 
		   PacketCount <= (others => '1');
		elsif(packet_edge = '1') then   
			PacketCount <= PacketCount + 1;
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
	RdAddrCount_tmp <= PacketCount & RdAddrCount_8b;
	RdAddrCount <= RdAddrCount_tmp + WrAddrCount;
	
-------------------------------------------------------------- 
	SRAMADDR <= WrAddrCount when capturing = '1' else RdAddrCount;
	
-------------------------------------------------------------- 
	SRAMWR_N <= not capturing;
	SRAMCS_N <= CLK;
	SRAMRD_N <= capturing;
	SRAMLE_N <= '0';	
	SRAMHE_N <= '0';
	
	SRAMDATA <= DUTDATA when capturing = '1' else (others => 'Z');

-------------------------------------------------------------- 
gen_PICDATA : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		PICDATA <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		PICDATA <= SRAMDATA;
	end if;  
end process;

END structure;
